Module: DS::Manifest
- Defined in:
- lib/ds/manifest.rb,
lib/ds/manifest/entry.rb,
lib/ds/manifest/manifest.rb,
lib/ds/manifest/constants.rb,
lib/ds/manifest/base_id_validator.rb,
lib/ds/manifest/manifest_validator.rb,
lib/ds/manifest/ds_csv_id_validator.rb,
lib/ds/manifest/simple_xml_id_validator.rb
Overview
Manifest comprises classes and a module (DS::Manifest::Constants) for encapsulating and validating a DS delivery manifest CSV.
The manifest CSV provides all information needed to ingest a set of source records. This information is detailed in Manifest and Entry.
The ManifestValidator validates that the Manifest is completed and well-formed, and that all records can be found the specified source diretory.
The valid Manifest is used by Converter::Converter to orchestrate mapping of source record data for the creation of the DS import CSV.
Defined Under Namespace
Modules: Constants Classes: BaseIdValidator, DsCsvIdValidator, Entry, Manifest, ManifestValidator, SimpleXmlIdValidator